<?php
namespace Ep\Woo\Plugin\Coupon;

use Ep\Woo\Email\CouponAdded;
use Ep\Wp\Plugin\PluginInterface;

/**
 * Class Notification
 *
 * @package Ep\Woo\Plugin\Coupon
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link http://gitlab.ep.local/ep-framework/woocommerce
 */
class Notifications implements PluginInterface
{
    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
        add_filter('woocommerce_email_actions', [$this, 'wc_email_actions']);
        add_filter('woocommerce_email_classes', [$this, 'wc_email_classes']);
        add_filter('woocommerce_locate_template', [$this, 'wc_locate_template'], 10, 2);
        add_filter('woocommerce_locate_core_template', [$this, 'wc_locate_core_template'], 10, 2);

        add_action('update_post_metadata', [$this, 'update_post_metadata'], 10, 5);
        add_action('woocommerce_coupon_added_notification', [$this, 'send_notification']);
    }

    public function update_post_metadata($null, $object_id, $meta_key, $meta_value, $prev_value)
    {
        if ('shop_coupon' === get_post_type($object_id) && 'customer_email' === $meta_key) {
            $prev_value = $prev_value ?: get_post_meta($object_id, $meta_key, true);
            $coupon = new \WC_Coupon($object_id);

            foreach ($meta_value as $email) {
                if (!in_array($email, $prev_value)) {
                    do_action('woocommerce_coupon_added', [
                        'user_email' => $email,
                        'coupon' => $coupon,
                    ]);
                }
            }
        }

        return $null;
    }

    public function send_notification($args)
    {
        if (!is_array($args)) {
            return;
        }

        $mailer = WC()->mailer();
        /** @var CouponAdded $mail */
        $mail = $mailer->emails['woo_coupon_added'];
        $mail->trigger($args['user_email'], $args['coupon']);
    }

    /**
     * @param array $actions
     * @return array
     */
    public function wc_email_actions(array $actions): array
    {
        $actions[] = 'woocommerce_coupon_added';
        return $actions;
    }

    /**
     * @param array $classes
     * @return array
     */
    public function wc_email_classes(array $classes): array
    {
        $classes['woo_coupon_added'] = new CouponAdded();
        return $classes;
    }

    public function wc_locate_core_template($core_file, string $template): string
    {
        if ('emails/customer-coupon-added.php' === $template) {
            return dirname(dirname(dirname(__DIR__))) . '/templates/' . $template;
        }
        return $core_file;
    }

    public function wc_locate_template(string $template, string $template_name): string
    {
        if ('emails/customer-coupon-added.php' === $template_name) {
            return $this->wc_locate_core_template('', $template_name);
        }
        return $template;
    }
}