<?php
namespace Ep\Woo\Plugin\Product;

use Ep\Wp\Plugin\PluginInterface;

final class AutoTable implements PluginInterface
{
    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
        add_filter('pre_do_shortcode_tag', [$this, 'do_table_shortcode'], 10, 4);
    }

    public function do_table_shortcode($false, $tag, $attr, $m)
    {
        if ($tag !== 'table' || ($attr['id'] ?? false) !== 'auto') {
            return $false;
        }

        $skus = array_values(array_filter(array_map('trim', explode("\n", strip_tags($m[5])))));
        /** @var \Ep\Gath\Plugin\Products $gathProducts */
        $gathProducts = apply_filters('ep_gath_products', null);
        $posts = $gathProducts->getProductsBySkus($skus);
        $product_ids = array_map(function(\WP_Post $post) { return $post->ID; }, $posts);
        /** @var \WC_Product[] $products */
        $products = array_map('wc_get_product', $product_ids);

        $table = [
            'id' => PHP_INT_MAX,
            'name' => 'Carparts Auto Table',
            'description' => '',
            'author' => 1,
            'last_modified' => date('Y-m-d H:i:s'),
            'data' => [[
                'ŠIFRA',
                'NAZIV ARTIKLA',
                'CIJENA',
                'AKCIJSKA CIJENA',
                'kom',
                'NARUČI ONLINE',
            ]],
            'options' => [
                'id' => PHP_INT_MAX,
                'column_widths' => '',
                'alternating_row_colors' => true,
                'row_hover' => true,
                'table_head' => true,
                'table_foot' => false,
                'first_column_th' => false,
                'print_name' => false,
                'print_name_position' => 'above',
                'print_description' => false,
                'print_description_position' => 'below',
                'cache_table_output' => true,
                'convert_line_breaks' => true,
                'extra_css_classes' => '',
                'use_datatables' => true,
                'datatables_sort' => true,
                'datatables_paginate' => false,
                'datatables_paginate_entries' => 10,
                'datatables_lengthchange' => true,
                'datatables_filter' => true,
                'datatables_info' => false,
                'datatables_scrollx' => false,
                'datatables_scrolly' => false,
                'datatables_custom_commands' => '',
                'datatables_locale' => 'hr',
                'show_rows' => '',
                'show_columns' => '',
                'hide_rows' => '',
                'hide_columns' => '',
                'cellspacing' => false,
                'cellpadding' => false,
                'border' => false,
                'shortcode_debug' => false,
                'html_id' => 'tablepress-auto',
            ],
            'visibility' => [
                'rows' => array_fill(0, count($products), 1),
                'columns' => array_fill(0, 6, 1),
            ],
        ];

        foreach ($products as $product) {
            $table['data'][] = [
                $product->get_sku(),
                $product->get_name(),
                strip_tags(wc_price($product->get_regular_price())),
                strip_tags(wc_price($product->get_sale_price())),
                $product->get_stock_quantity(),
                sprintf('<a href="%s">%s</a>', get_permalink($product->get_id()), 'Kupi na webshopu'),
            ];
        }

        /** @var \TablePress_Frontend_Controller $controller */
        $controller = \TablePress::$controller;
        /** @noinspection PhpUnhandledExceptionInspection */
        $reflController = new \ReflectionClass($controller);
        $shown_tables = $reflController->getProperty('shown_tables');
        $shown_tables->setAccessible(true);
        $shown_tables->setValue($controller, [
            PHP_INT_MAX => [
                'instances' => [
                    'tablepress-auto' => $table['options'],
                ],
            ],
        ]);

        /** @var \TablePress_Render $render */
        $renderer = \TablePress::load_class('TablePress_Render', 'class-render.php', 'classes');
        $renderer->set_input($table, $table['options']);

        $js_file = 'js/jquery.datatables.min.js';
        $js_url = plugins_url( $js_file, TABLEPRESS__FILE__ );
        /**
         * Filter the URL from which the DataTables JavaScript library file is loaded.
         *
         * @since 1.0.0
         *
         * @param string $js_url  URL of the DataTables JS library file.
         * @param string $js_file Path and file name of the DataTables JS library file.
         */
        $js_url = apply_filters( 'tablepress_datatables_js_url', $js_url, $js_file );
        wp_enqueue_script( 'tablepress-datatables', $js_url, array( 'jquery-core' ), \TablePress::version, true );


        return $renderer->get_output();
    }
}