<?php
namespace Ep\Woo\Plugin\Product;

use Ep\Wp\Plugin\PluginInterface;
use WC_Product;
use WP_Term;

/**
 * Omogućava dodijeljivanje backorder statusa na razini branda.
 *https://wptest.ep.hr/aquashop
 * @package Ep\Woo\Plugin\Brand
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class Backorders implements PluginInterface
{
    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
        add_filter('woocommerce_get_availability_text', [$this, 'wc_get_availability_text'], 10, 2);
        add_filter('woocommerce_get_settings_products', [$this, 'wc_get_settings_products'], 10, 2);
    }

    /**
     * Vraća tekst kada proizvoda nema na zalihi.
     *
     * @param string $availability
     * @param WC_Product $product
     *
     * @return string
     */
    public function wc_get_availability_text(string $availability, WC_Product $product): string
    {
        if (0 === $product->get_stock_quantity('')) {
            $specialBrands = array_map('absint', get_option('ep_woo_backorder_brands', []) ?: []);
            $brands = wp_get_post_terms($product->get_id(), 'product_brand') ?: [];
            $brandIds = array_map(function(WP_Term $term) {
                return $term->term_id;
            }, $brands);


            if (count(array_intersect($specialBrands, $brandIds)) > 0) {
                return get_option('ep_woo_backorder_brands_text');
            } else {
                return get_option('ep_woo_backorder_text');
            }
        }

        return $availability;
    }

    /**
     * Dodaje postavke u tab 'Proizvodi', sekciju 'Inventar'
     *
     * @param array $settings
     * @param string $section
     *
     * @return array
     */
    public function wc_get_settings_products(array $settings, string $section): array
    {
        if ('inventory' === $section) {
            $last = array_pop($settings);
            $settings = array_merge($settings, $this->getSettingsFields());
            array_push($settings, $last);
        }

        return $settings;
    }

    /**
     * Vraća polja za postavke.
     *
     * @return array
     */
    private function getSettingsFields()
    {
        static $brands;

        if (!isset($brands)) {
            $brands = [];
            /** @var WP_Term[] $terms */
            $terms = get_terms([
                'taxonomy' => 'product_brand',
                'hide_empty' => false,
            ]);

            foreach ($terms as $term) {
                $brands[$term->term_id] = $term->name;
            }
        }

        return [[
            'id' => 'ep_woo_backorder_text',
            'name' => 'Tekst koji se vidi na proizvodima kojih nema na zalihi',
            'type' => 'text'
        ], [
            'id' => 'ep_woo_backorder_brands',
            'name' => 'Brandovi proizvoda za koje prikazujemo poseban tekst',
            'type' => 'multiselect',
            'options' => $brands,
            'class' => 'wc-enhanced-select',
        ], [
            'id' => 'ep_woo_backorder_brands_text',
            'name' => 'Tekst koji prikazujemo za proizvode kojih nema na zalihi, a od jednog od gore navedenih proizvođača su',
            'type' => 'text',
        ]];
    }
}