<?php
namespace Ep\Woo\Plugin\Shipping;

use Ep\Wp\Plugin\PluginInterface;

/**
 * Sakriva metode dostave ovisno o kriterijima.
 *
 * - Ako je dostupna besplatna dostava, sakriva sve ostale plaćene metode
 * - Ako košarica sadrži gume, sakrivamo lokalno preuzimanje
 *
 * @package Ep\Woo\Plugin\Shipping
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/woocommerce
 */
class HideMethods implements PluginInterface
{
    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
        add_filter('woocommerce_package_rates', [$this, 'wc_package_rates']);
    }

    /**
     * Filtrira metode dostave
     *
     * @param \WC_Shipping_Rate[] $rates
     *
     * @return \WC_Shipping_Rate[]
     */
    public function wc_package_rates($rates)
    {
        $hasFreeShipping = false;
        $flatRateKeys = [];
        $localPickupKeys = [];

        foreach ($rates as $key => $rate) {
            switch ($rate->get_method_id()) {
                case 'flat_rate':
                    $flatRateKeys[] = $key;
                    break;
                case 'free_shipping':
                    $hasFreeShipping = true;
                    break;
                case 'local_pickup':
                    $localPickupKeys[] = $key;
                    break;
            }
        }

        if ($hasFreeShipping) {
            foreach ($flatRateKeys as $key) {
                unset($rates[$key]);
            }
        }

        $cart = WC()->cart;
        $product_types = array_map(function($item) use ($cart) {
            return wc_get_product($item['product_id'])->get_type();
        }, $cart->get_cart());

        if (in_array('tire', $product_types)) {
            foreach ($localPickupKeys as $key) {
                unset($rates[$key]);
            }
        }

        return $rates;
    }
}