<?php
namespace Ep\Woo\Product;

/**
 * Proizvod.
 *
 * @package Ep\Woo\Product
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
interface ProductInterface
{
    /**
     * Vraća id ovog proizvoda.
     *
     * @return int
     */
    public function getId(): int;

    /**
     * Najveća količina u kojoj je moguće kupiti ovaj proizvod.
     *
     * @return int
     */
    public function getMaxPurchaseQuantity(): int;

    /**
     * Naziv ovog proizvoda.
     *
     * @return string
     */
    public function getName(): string;

    /**
     * Najmanja količina u kojoj je moguće kupiti ovaj proizvod.
     *
     * @return int
     */
    public function getMinPurchaseQuantity(): int;

    /**
     * Vraća true ako je ovaj proizvod na akciji.
     *
     * @return bool
     */
    public function isOnSale(): bool;

    /**
     * Vraća iznos popusta u postotku.
     *
     * @return float
     */
    public function getDiscountPercentage(): int;

    /**
     * Vraća cijenu ovog proizvoda.
     *
     * @return float
     */
    public function getPrice(): float;

    /**
     * Da li je ovaj proizvod moguće kupiti.
     *
     * @return bool
     */
    public function isPurchasable(): bool;

    /**
     * Vraća korake u kojima je moguće kupiti proizvod.
     *
     * @return int
     */
    public function getPurchaseStep(): int;

    /**
     * Vraća redovnu cijenu ako je proizvod na akciji.
     *
     * @return float
     */
    public function getRegularPrice(): float;

    /**
     * Vraća akcijsku cijenu proizvoda.
     *
     * @return float
     */
    public function getSalePrice(): float;

    /**
     * Vraća šifru ovog proizvoda.
     *
     * @return string
     */
    public function getSku(): string;

    /**
     * Vraća stanje zalihe ovog proizvoda.
     *
     * @return int
     */
    public function getStock(): int;

    /**
     * Vraća vrstu ovog proizvoda.
     *
     * @return string
     */
    public function getType(): string;

    /**
     * Vraca brand
     *
     * @return array
     */
    public function getBrand();

    /**
     * Da li je ovaj proizvod vidljiv u katalogu.
     *
     * @return bool
     */
    public function isVisible(): bool;
}